import { Router } from '@angular/router';
import { URLSearchParams } from '@angular/http';
import { throwError, Subject } from 'rxjs';
import { LocalStorageService, TrycatchLocalStorageModule } from '@trycatch/local-storage';
import { Injectable, Optional, NgModule, SkipSelf } from '@angular/core';
import { HttpClient, HttpHeaders, HttpResponse, HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { catchError, map, finalize, tap, timeout } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const HTTP_STATUS_CODE = {
    OK: 200,
    CREATED: 201,
    NOT_FOUND: 404,
    UNAUTHROISED: 401,
    ACCESS_FORBIDDEN: 403,
    SERVER_ERROR: 500,
};
HTTP_STATUS_CODE[HTTP_STATUS_CODE.OK] = 'OK';
HTTP_STATUS_CODE[HTTP_STATUS_CODE.CREATED] = 'CREATED';
HTTP_STATUS_CODE[HTTP_STATUS_CODE.NOT_FOUND] = 'NOT_FOUND';
HTTP_STATUS_CODE[HTTP_STATUS_CODE.UNAUTHROISED] = 'UNAUTHROISED';
HTTP_STATUS_CODE[HTTP_STATUS_CODE.ACCESS_FORBIDDEN] = 'ACCESS_FORBIDDEN';
HTTP_STATUS_CODE[HTTP_STATUS_CODE.SERVER_ERROR] = 'SERVER_ERROR';
/** @enum {number} */
const HTTP_METHOD = {
    GET: 0,
    POST: 1,
    PATCH: 2,
    PUT: 3,
    DELETE: 4,
};
HTTP_METHOD[HTTP_METHOD.GET] = 'GET';
HTTP_METHOD[HTTP_METHOD.POST] = 'POST';
HTTP_METHOD[HTTP_METHOD.PATCH] = 'PATCH';
HTTP_METHOD[HTTP_METHOD.PUT] = 'PUT';
HTTP_METHOD[HTTP_METHOD.DELETE] = 'DELETE';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class WebApiConfig {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class WebApiService {
    /**
     * @param {?} http
     * @param {?} localStorage
     * @param {?} router
     * @param {?} config
     */
    constructor(http, localStorage, router, config) {
        this.http = http;
        this.localStorage = localStorage;
        this.router = router;
        this.config = (/** @type {?} */ ({}));
        this.progressEvent = new Subject();
        this.progressEvent$ = this.progressEvent.asObservable();
        this.errorSubject = new Subject();
        this.webApiError = this.errorSubject.asObservable();
        if (config) {
            this.config = config;
        }
    }
    /**
     * @template T
     * @param {?} api
     * @param {?=} useBaseUrl
     * @param {?=} extraHeaders
     * @return {?}
     */
    get(api, useBaseUrl = true, extraHeaders) {
        /** @type {?} */
        let httpOptions;
        if (extraHeaders) {
            /** @type {?} */
            const headers = new HttpHeaders(extraHeaders);
            httpOptions = {
                headers: headers
            };
        }
        return this.http
            .get(this.fullUrl(api, useBaseUrl), httpOptions)
            .pipe(catchError((/**
         * @param {?} errorResponse
         * @param {?} caught
         * @return {?}
         */
        (errorResponse, caught) => {
            console.log("catch an error in get call");
            return this.errorHandler(errorResponse);
        })));
    }
    /**
     * @private
     * @template T
     * @param {?} api
     * @param {?} body
     * @param {?} options
     * @param {?=} useBaseUrl
     * @return {?}
     */
    postUrlEncoded(api, body, options, useBaseUrl = true) {
        /** @type {?} */
        const payload = this.transformPayloadToFormUrlEncoded(body).toString();
        options["headers"] = {
            "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8"
        };
        return this.http
            .post(this.fullUrl(api, useBaseUrl), payload, options)
            .pipe(catchError((/**
         * @param {?} errorResponse
         * @param {?} caught
         * @return {?}
         */
        (errorResponse, caught) => {
            return this.errorHandler(errorResponse);
        })));
    }
    /**
     * @template T
     * @param {?} api
     * @param {?} body
     * @param {?=} useBaseUrl
     * @param {?=} responseType
     * @param {?=} contentType
     * @return {?}
     */
    post(api, body, useBaseUrl = true, responseType, contentType) {
        /** @type {?} */
        let options = {};
        if (responseType) {
            options = { responseType: responseType };
        }
        if (contentType) {
            if (contentType === "form-urlencoded") {
                return this.postUrlEncoded(api, body, options);
            }
        }
        else {
            return this.http
                .post(this.fullUrl(api, useBaseUrl), body, options)
                .pipe(catchError((/**
             * @param {?} errorResponse
             * @param {?} caught
             * @return {?}
             */
            (errorResponse, caught) => {
                return this.errorHandler(errorResponse);
            })));
        }
    }
    // public post<T>(
    // 	api,
    // 	body,
    // 	useBaseUrl = true,
    // 	responseType?: string,
    // 	contentType?: string
    // ): Observable<T> {
    // 	let options = {};
    // 	if (responseType) {
    // 		options = { responseType: responseType };
    // 	}
    // 	if (contentType) {
    // 		if (contentType === "form-urlencoded") {
    // 			options = {
    // 				headers: {
    // 					"Content-Type":
    // 						"application/x-www-form-urlencoded; charset=UTF-8"
    // 				}
    // 			};
    // 			body = body.toString();
    // 		}
    // 	}
    // 	return this.http
    // 		.post<T>(this.fullUrl(api, useBaseUrl), body, options)
    // 		.pipe(
    // 			catchError(
    // 				(errorResponse: ApiError, caught: Observable<any>) => {
    // 					return this.errorHandler(errorResponse);
    // 				}
    // 			)
    // 		);
    // }
    /**
     * @private
     * @param {?} payload
     * @param {?=} parentKey
     * @return {?}
     */
    transformPayloadToFormUrlEncoded(payload, parentKey = null) {
        /** @type {?} */
        const urlEncodedPayload = new URLSearchParams();
        Object.keys(payload).forEach((/**
         * @param {?} property
         * @return {?}
         */
        property => {
            /** @type {?} */
            let val = payload[property];
            if (!val) {
                val = "";
            }
            /** @type {?} */
            let key = property;
            if (parentKey) {
                key = parentKey + "[" + key + "]";
            }
            if (typeof val !== "object") {
                urlEncodedPayload.set(key, val);
            }
            else {
                /** @type {?} */
                const params = this.transformPayloadToFormUrlEncoded(val, key);
                params.paramsMap.forEach((/**
                 * @param {?} v
                 * @param {?} k
                 * @return {?}
                 */
                (v, k) => urlEncodedPayload.set(k, v[0])));
            }
        }));
        return urlEncodedPayload;
    }
    /**
     * @template T
     * @param {?} api
     * @param {?} item
     * @param {?=} useBaseUrl
     * @return {?}
     */
    update(api, item, useBaseUrl = true) {
        /** @type {?} */
        const url = this.fullUrl(api, useBaseUrl);
        return this.http.put(url, item).pipe(catchError((/**
         * @param {?} errorResponse
         * @param {?} caught
         * @return {?}
         */
        (errorResponse, caught) => {
            return this.errorHandler(errorResponse);
        })));
    }
    /**
     * @template T
     * @param {?} api
     * @param {?} item
     * @param {?=} useBaseUrl
     * @return {?}
     */
    delete(api, item, useBaseUrl = true) {
        /** @type {?} */
        const url = this.fullUrl(api, useBaseUrl);
        return this.http.delete(url).pipe(catchError((/**
         * @param {?} errorResponse
         * @param {?} caught
         * @return {?}
         */
        (errorResponse, caught) => {
            return this.errorHandler(errorResponse);
        })));
    }
    // public uploadFile(
    // 	api: string,
    // 	file: File,
    // 	fileName: string,
    // 	data: string,
    // 	useBaseUrl = true
    // ): Observable<any> {
    // 	const formData = new FormData();
    // 	formData.append("data", data);
    // 	formData.append("filename", file, fileName);
    // 	return this.http.post(this.fullUrl(api, useBaseUrl), formData, {
    // 		headers: {}
    // 	});
    // }
    // public uploadFileWithProgressEvent(
    // 	api: string,
    // 	file: File,
    // 	fileName: string,
    // 	useBaseUrl = true
    // ): Observable<any> {
    // 	return Observable.create(observer => {
    // 		const formData: FormData = new FormData(),
    // 			xhr: XMLHttpRequest = new XMLHttpRequest();
    // 		formData.append("files", file, fileName);
    // 		xhr.onreadystatechange = () => {
    // 			if (xhr.readyState === 4) {
    // 				if (xhr.status === 200) {
    // 					observer.next(JSON.parse(xhr.response));
    // 					observer.complete();
    // 				} else {
    // 					observer.error(xhr.response);
    // 				}
    // 			}
    // 		};
    // 		xhr.upload.onprogress = event => {
    // 			const progress = Math.round(event.loaded / event.total * 100);
    // 			this.progressEvent.next(progress);
    // 		};
    // 		xhr.open("POST", this.fullUrl(api, useBaseUrl), true);
    // 		xhr.send(formData);
    // 	});
    // }
    // private functions
    /**
     * @private
     * @param {?} api
     * @param {?} useBaseUrl
     * @return {?}
     */
    fullUrl(api, useBaseUrl) {
        return this.config.baseUrl && useBaseUrl
            ? this.config.baseUrl + api
            : api;
    }
    /**
     * @param {?} apiError
     * @return {?}
     */
    errorHandler(apiError) {
        if (apiError.apiUrl) {
            this.errorSubject.next(apiError);
        }
        return throwError(apiError);
    }
}
WebApiService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
WebApiService.ctorParameters = () => [
    { type: HttpClient },
    { type: LocalStorageService },
    { type: Router },
    { type: WebApiConfig, decorators: [{ type: Optional }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class JsonFileService {
    /**
     * @param {?} http
     */
    constructor(http) {
        this.http = http;
        this.errorSubject = new Subject();
        this.jsonFileError = this.errorSubject.asObservable();
    }
    /**
     * @template T
     * @param {?} jsonFileUrl
     * @return {?}
     */
    get(jsonFileUrl) {
        return this.getRemoteJsonFile(jsonFileUrl);
    }
    /**
     * @private
     * @template T
     * @param {?} jsonFileUrl
     * @return {?}
     */
    getRemoteJsonFile(jsonFileUrl) {
        return this.http.get(jsonFileUrl).pipe(map((/**
         * @param {?} result
         * @return {?}
         */
        result => (/** @type {?} */ (result)))), catchError((/**
         * @param {?} error
         * @return {?}
         */
        error => this.errorResponseHandler(error, jsonFileUrl))));
    }
    /**
     * @private
     * @param {?} error
     * @param {?} jsonFileUrl
     * @return {?}
     */
    errorResponseHandler(error, jsonFileUrl) {
        /** @type {?} */
        const jsonFileError = (/** @type {?} */ ({}));
        jsonFileError.fileUrl = jsonFileUrl;
        jsonFileError.error = error;
        return throwError(jsonFileError);
    }
}
JsonFileService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
JsonFileService.ctorParameters = () => [
    { type: HttpClient }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TimeLoggingInterceptor {
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        /** @type {?} */
        const started = Date.now();
        /** @type {?} */
        let ok;
        return next.handle(request).pipe(tap((
        // Succeeds when there is a response; ignore other events
        /**
         * @param {?} event
         * @return {?}
         */
        event => (ok = event instanceof HttpResponse ? "succeeded" : "")), (
        // Operation failed; error is an HttpErrorResponse
        /**
         * @param {?} error
         * @return {?}
         */
        error => (ok = "failed"))), 
        // Log when response observable either completes or errors
        finalize((/**
         * @return {?}
         */
        () => {
            /** @type {?} */
            const elapsed = Date.now() - started;
            /** @type {?} */
            const msg = `%c ${request.method} "${request.urlWithParams}" ${ok} in ${elapsed} ms.`;
            console.log(msg, "color: #c01025");
        })));
    }
}
TimeLoggingInterceptor.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ErrorHandlingInterceptor {
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        return next.handle(request).pipe(catchError((/**
         * @param {?} errorResponse
         * @param {?} caught
         * @return {?}
         */
        (errorResponse, caught) => {
            console.log("raw error : ", errorResponse);
            return this.errorHandler(request, errorResponse);
        })));
    }
    /**
     * @private
     * @param {?} request
     * @param {?} errorResponse
     * @return {?}
     */
    errorHandler(request, errorResponse) {
        /** @type {?} */
        const apiError = (/** @type {?} */ ({
            method: request.method.toUpperCase(),
            apiUrl: request.url,
            error: errorResponse
        }));
        switch ((/** @type {?} */ (errorResponse.status))) {
            case HTTP_STATUS_CODE.NOT_FOUND:
                return this.error404Handler(apiError);
            case HTTP_STATUS_CODE.SERVER_ERROR:
                return this.error500Handler(apiError);
            default:
                return throwError(apiError);
        }
    }
    /**
     * @private
     * @param {?} apiError
     * @return {?}
     */
    error404Handler(apiError) {
        return throwError(apiError);
    }
    /**
     * @private
     * @param {?} apiError
     * @return {?}
     */
    error500Handler(apiError) {
        console.log("Please handle 500 error - internal server error");
        return throwError(apiError);
    }
}
ErrorHandlingInterceptor.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ContentTypeInterceptor {
    constructor() { }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        if (request.method === "POST" && !(request.body instanceof FormData)) {
            /** @type {?} */
            const headerSet = !!request.headers.get("Content-Type");
            if (!headerSet) {
                request = request.clone({
                    setHeaders: {
                        "Content-Type": "application/json"
                    }
                });
            }
        }
        return next.handle(request);
    }
}
ContentTypeInterceptor.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ContentTypeInterceptor.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TokenInterceptor {
    /**
     * @param {?} localStorage
     * @param {?} config
     */
    constructor(localStorage, config) {
        this.localStorage = localStorage;
        this.config = config;
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        /** @type {?} */
        const token = this.localStorage.get(this.config.tokenKey);
        if (token) {
            request = request.clone({
                setHeaders: {
                    token: token
                }
            });
        }
        return next.handle(request);
    }
}
TokenInterceptor.decorators = [
    { type: Injectable }
];
/** @nocollapse */
TokenInterceptor.ctorParameters = () => [
    { type: LocalStorageService },
    { type: WebApiConfig, decorators: [{ type: Optional }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DEFAULT_TIMEOUT = 10000 * 2.5;
class TimeOutInterceptor {
    /**
     * @param {?} config
     */
    constructor(config) {
        this.config = config;
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        /** @type {?} */
        const timeOutTime = this.config.timeout || DEFAULT_TIMEOUT;
        return next.handle(request).pipe(timeout(timeOutTime));
    }
}
TimeOutInterceptor.decorators = [
    { type: Injectable }
];
/** @nocollapse */
TimeOutInterceptor.ctorParameters = () => [
    { type: WebApiConfig, decorators: [{ type: Optional }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ResultHandlingInterceptor {
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        return next.handle(request).pipe(map((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            if (event instanceof HttpResponse &&
                request.url.indexOf(".json") < 0) {
                if (event.body) {
                    return event.clone({ body: event.body.result });
                }
            }
            return event;
        })));
    }
}
ResultHandlingInterceptor.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const WebApiInterceptorProviders = [
    {
        provide: HTTP_INTERCEPTORS,
        useClass: TimeLoggingInterceptor,
        multi: true
    },
    {
        provide: HTTP_INTERCEPTORS,
        useClass: ResultHandlingInterceptor,
        multi: true
    },
    {
        provide: HTTP_INTERCEPTORS,
        useClass: ErrorHandlingInterceptor,
        multi: true
    },
    { provide: HTTP_INTERCEPTORS, useClass: TimeOutInterceptor, multi: true },
    { provide: HTTP_INTERCEPTORS, useClass: TokenInterceptor, multi: true },
    {
        provide: HTTP_INTERCEPTORS,
        useClass: ContentTypeInterceptor,
        multi: true
    } // adding content type to headers
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TryCatchWebApiModule {
    /**
     * @param {?} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error("TryCatch Web Api Module is already loaded. Import it in the AppModule only");
        }
    }
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: TryCatchWebApiModule,
            providers: [
                WebApiService,
                { provide: WebApiConfig, useValue: config }
            ]
        };
    }
}
TryCatchWebApiModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    HttpClientModule,
                    TrycatchLocalStorageModule.forRoot({
                        storageType: "local"
                    })
                ],
                providers: [WebApiInterceptorProviders, WebApiService, JsonFileService]
            },] }
];
/** @nocollapse */
TryCatchWebApiModule.ctorParameters = () => [
    { type: TryCatchWebApiModule, decorators: [{ type: Optional }, { type: SkipSelf }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { HTTP_STATUS_CODE, HTTP_METHOD, WebApiConfig, WebApiService, JsonFileService, TryCatchWebApiModule, ContentTypeInterceptor as ɵg, ErrorHandlingInterceptor as ɵd, ResultHandlingInterceptor as ɵc, TimeLoggingInterceptor as ɵb, TimeOutInterceptor as ɵe, TokenInterceptor as ɵf, WebApiInterceptorProviders as ɵa };

//# sourceMappingURL=trycatch-web-api.js.map