import { Router } from '@angular/router';
import { URLSearchParams } from '@angular/http';
import { throwError, Subject } from 'rxjs';
import { LocalStorageService, TrycatchLocalStorageModule } from '@trycatch/local-storage';
import { Injectable, Optional, NgModule, SkipSelf } from '@angular/core';
import { HttpClient, HttpHeaders, HttpResponse, HTTP_INTERCEPTORS, HttpClientModule } from '@angular/common/http';
import { catchError, map, finalize, tap, timeout } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
var HTTP_STATUS_CODE = {
    OK: 200,
    CREATED: 201,
    NOT_FOUND: 404,
    UNAUTHROISED: 401,
    ACCESS_FORBIDDEN: 403,
    SERVER_ERROR: 500,
};
HTTP_STATUS_CODE[HTTP_STATUS_CODE.OK] = 'OK';
HTTP_STATUS_CODE[HTTP_STATUS_CODE.CREATED] = 'CREATED';
HTTP_STATUS_CODE[HTTP_STATUS_CODE.NOT_FOUND] = 'NOT_FOUND';
HTTP_STATUS_CODE[HTTP_STATUS_CODE.UNAUTHROISED] = 'UNAUTHROISED';
HTTP_STATUS_CODE[HTTP_STATUS_CODE.ACCESS_FORBIDDEN] = 'ACCESS_FORBIDDEN';
HTTP_STATUS_CODE[HTTP_STATUS_CODE.SERVER_ERROR] = 'SERVER_ERROR';
/** @enum {number} */
var HTTP_METHOD = {
    GET: 0,
    POST: 1,
    PATCH: 2,
    PUT: 3,
    DELETE: 4,
};
HTTP_METHOD[HTTP_METHOD.GET] = 'GET';
HTTP_METHOD[HTTP_METHOD.POST] = 'POST';
HTTP_METHOD[HTTP_METHOD.PATCH] = 'PATCH';
HTTP_METHOD[HTTP_METHOD.PUT] = 'PUT';
HTTP_METHOD[HTTP_METHOD.DELETE] = 'DELETE';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var WebApiConfig = /** @class */ (function () {
    function WebApiConfig() {
    }
    return WebApiConfig;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var WebApiService = /** @class */ (function () {
    function WebApiService(http, localStorage, router, config) {
        this.http = http;
        this.localStorage = localStorage;
        this.router = router;
        this.config = (/** @type {?} */ ({}));
        this.progressEvent = new Subject();
        this.progressEvent$ = this.progressEvent.asObservable();
        this.errorSubject = new Subject();
        this.webApiError = this.errorSubject.asObservable();
        if (config) {
            this.config = config;
        }
    }
    /**
     * @template T
     * @param {?} api
     * @param {?=} useBaseUrl
     * @param {?=} extraHeaders
     * @return {?}
     */
    WebApiService.prototype.get = /**
     * @template T
     * @param {?} api
     * @param {?=} useBaseUrl
     * @param {?=} extraHeaders
     * @return {?}
     */
    function (api, useBaseUrl, extraHeaders) {
        var _this = this;
        if (useBaseUrl === void 0) { useBaseUrl = true; }
        /** @type {?} */
        var httpOptions;
        if (extraHeaders) {
            /** @type {?} */
            var headers = new HttpHeaders(extraHeaders);
            httpOptions = {
                headers: headers
            };
        }
        return this.http
            .get(this.fullUrl(api, useBaseUrl), httpOptions)
            .pipe(catchError((/**
         * @param {?} errorResponse
         * @param {?} caught
         * @return {?}
         */
        function (errorResponse, caught) {
            console.log("catch an error in get call");
            return _this.errorHandler(errorResponse);
        })));
    };
    /**
     * @private
     * @template T
     * @param {?} api
     * @param {?} body
     * @param {?} options
     * @param {?=} useBaseUrl
     * @return {?}
     */
    WebApiService.prototype.postUrlEncoded = /**
     * @private
     * @template T
     * @param {?} api
     * @param {?} body
     * @param {?} options
     * @param {?=} useBaseUrl
     * @return {?}
     */
    function (api, body, options, useBaseUrl) {
        var _this = this;
        if (useBaseUrl === void 0) { useBaseUrl = true; }
        /** @type {?} */
        var payload = this.transformPayloadToFormUrlEncoded(body).toString();
        options["headers"] = {
            "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8"
        };
        return this.http
            .post(this.fullUrl(api, useBaseUrl), payload, options)
            .pipe(catchError((/**
         * @param {?} errorResponse
         * @param {?} caught
         * @return {?}
         */
        function (errorResponse, caught) {
            return _this.errorHandler(errorResponse);
        })));
    };
    /**
     * @template T
     * @param {?} api
     * @param {?} body
     * @param {?=} useBaseUrl
     * @param {?=} responseType
     * @param {?=} contentType
     * @return {?}
     */
    WebApiService.prototype.post = /**
     * @template T
     * @param {?} api
     * @param {?} body
     * @param {?=} useBaseUrl
     * @param {?=} responseType
     * @param {?=} contentType
     * @return {?}
     */
    function (api, body, useBaseUrl, responseType, contentType) {
        var _this = this;
        if (useBaseUrl === void 0) { useBaseUrl = true; }
        /** @type {?} */
        var options = {};
        if (responseType) {
            options = { responseType: responseType };
        }
        if (contentType) {
            if (contentType === "form-urlencoded") {
                return this.postUrlEncoded(api, body, options);
            }
        }
        else {
            return this.http
                .post(this.fullUrl(api, useBaseUrl), body, options)
                .pipe(catchError((/**
             * @param {?} errorResponse
             * @param {?} caught
             * @return {?}
             */
            function (errorResponse, caught) {
                return _this.errorHandler(errorResponse);
            })));
        }
    };
    // public post<T>(
    // 	api,
    // 	body,
    // 	useBaseUrl = true,
    // 	responseType?: string,
    // 	contentType?: string
    // ): Observable<T> {
    // 	let options = {};
    // 	if (responseType) {
    // 		options = { responseType: responseType };
    // 	}
    // 	if (contentType) {
    // 		if (contentType === "form-urlencoded") {
    // 			options = {
    // 				headers: {
    // 					"Content-Type":
    // 						"application/x-www-form-urlencoded; charset=UTF-8"
    // 				}
    // 			};
    // 			body = body.toString();
    // 		}
    // 	}
    // 	return this.http
    // 		.post<T>(this.fullUrl(api, useBaseUrl), body, options)
    // 		.pipe(
    // 			catchError(
    // 				(errorResponse: ApiError, caught: Observable<any>) => {
    // 					return this.errorHandler(errorResponse);
    // 				}
    // 			)
    // 		);
    // }
    // public post<T>(
    // 	api,
    // 	body,
    // 	useBaseUrl = true,
    // 	responseType?: string,
    // 	contentType?: string
    // ): Observable<T> {
    // 	let options = {};
    // 	if (responseType) {
    // 		options = { responseType: responseType };
    // 	}
    // 	if (contentType) {
    // 		if (contentType === "form-urlencoded") {
    // 			options = {
    // 				headers: {
    // 					"Content-Type":
    // 						"application/x-www-form-urlencoded; charset=UTF-8"
    // 				}
    // 			};
    // 			body = body.toString();
    // 		}
    // 	}
    // 	return this.http
    // 		.post<T>(this.fullUrl(api, useBaseUrl), body, options)
    // 		.pipe(
    // 			catchError(
    // 				(errorResponse: ApiError, caught: Observable<any>) => {
    // 					return this.errorHandler(errorResponse);
    // 				}
    // 			)
    // 		);
    // }
    /**
     * @private
     * @param {?} payload
     * @param {?=} parentKey
     * @return {?}
     */
    WebApiService.prototype.transformPayloadToFormUrlEncoded = 
    // public post<T>(
    // 	api,
    // 	body,
    // 	useBaseUrl = true,
    // 	responseType?: string,
    // 	contentType?: string
    // ): Observable<T> {
    // 	let options = {};
    // 	if (responseType) {
    // 		options = { responseType: responseType };
    // 	}
    // 	if (contentType) {
    // 		if (contentType === "form-urlencoded") {
    // 			options = {
    // 				headers: {
    // 					"Content-Type":
    // 						"application/x-www-form-urlencoded; charset=UTF-8"
    // 				}
    // 			};
    // 			body = body.toString();
    // 		}
    // 	}
    // 	return this.http
    // 		.post<T>(this.fullUrl(api, useBaseUrl), body, options)
    // 		.pipe(
    // 			catchError(
    // 				(errorResponse: ApiError, caught: Observable<any>) => {
    // 					return this.errorHandler(errorResponse);
    // 				}
    // 			)
    // 		);
    // }
    /**
     * @private
     * @param {?} payload
     * @param {?=} parentKey
     * @return {?}
     */
    function (payload, parentKey) {
        var _this = this;
        if (parentKey === void 0) { parentKey = null; }
        /** @type {?} */
        var urlEncodedPayload = new URLSearchParams();
        Object.keys(payload).forEach((/**
         * @param {?} property
         * @return {?}
         */
        function (property) {
            /** @type {?} */
            var val = payload[property];
            if (!val) {
                val = "";
            }
            /** @type {?} */
            var key = property;
            if (parentKey) {
                key = parentKey + "[" + key + "]";
            }
            if (typeof val !== "object") {
                urlEncodedPayload.set(key, val);
            }
            else {
                /** @type {?} */
                var params = _this.transformPayloadToFormUrlEncoded(val, key);
                params.paramsMap.forEach((/**
                 * @param {?} v
                 * @param {?} k
                 * @return {?}
                 */
                function (v, k) {
                    return urlEncodedPayload.set(k, v[0]);
                }));
            }
        }));
        return urlEncodedPayload;
    };
    /**
     * @template T
     * @param {?} api
     * @param {?} item
     * @param {?=} useBaseUrl
     * @return {?}
     */
    WebApiService.prototype.update = /**
     * @template T
     * @param {?} api
     * @param {?} item
     * @param {?=} useBaseUrl
     * @return {?}
     */
    function (api, item, useBaseUrl) {
        var _this = this;
        if (useBaseUrl === void 0) { useBaseUrl = true; }
        /** @type {?} */
        var url = this.fullUrl(api, useBaseUrl);
        return this.http.put(url, item).pipe(catchError((/**
         * @param {?} errorResponse
         * @param {?} caught
         * @return {?}
         */
        function (errorResponse, caught) {
            return _this.errorHandler(errorResponse);
        })));
    };
    /**
     * @template T
     * @param {?} api
     * @param {?} item
     * @param {?=} useBaseUrl
     * @return {?}
     */
    WebApiService.prototype.delete = /**
     * @template T
     * @param {?} api
     * @param {?} item
     * @param {?=} useBaseUrl
     * @return {?}
     */
    function (api, item, useBaseUrl) {
        var _this = this;
        if (useBaseUrl === void 0) { useBaseUrl = true; }
        /** @type {?} */
        var url = this.fullUrl(api, useBaseUrl);
        return this.http.delete(url).pipe(catchError((/**
         * @param {?} errorResponse
         * @param {?} caught
         * @return {?}
         */
        function (errorResponse, caught) {
            return _this.errorHandler(errorResponse);
        })));
    };
    // public uploadFile(
    // 	api: string,
    // 	file: File,
    // 	fileName: string,
    // 	data: string,
    // 	useBaseUrl = true
    // ): Observable<any> {
    // 	const formData = new FormData();
    // 	formData.append("data", data);
    // 	formData.append("filename", file, fileName);
    // 	return this.http.post(this.fullUrl(api, useBaseUrl), formData, {
    // 		headers: {}
    // 	});
    // }
    // public uploadFileWithProgressEvent(
    // 	api: string,
    // 	file: File,
    // 	fileName: string,
    // 	useBaseUrl = true
    // ): Observable<any> {
    // 	return Observable.create(observer => {
    // 		const formData: FormData = new FormData(),
    // 			xhr: XMLHttpRequest = new XMLHttpRequest();
    // 		formData.append("files", file, fileName);
    // 		xhr.onreadystatechange = () => {
    // 			if (xhr.readyState === 4) {
    // 				if (xhr.status === 200) {
    // 					observer.next(JSON.parse(xhr.response));
    // 					observer.complete();
    // 				} else {
    // 					observer.error(xhr.response);
    // 				}
    // 			}
    // 		};
    // 		xhr.upload.onprogress = event => {
    // 			const progress = Math.round(event.loaded / event.total * 100);
    // 			this.progressEvent.next(progress);
    // 		};
    // 		xhr.open("POST", this.fullUrl(api, useBaseUrl), true);
    // 		xhr.send(formData);
    // 	});
    // }
    // private functions
    // public uploadFile(
    // 	api: string,
    // 	file: File,
    // 	fileName: string,
    // 	data: string,
    // 	useBaseUrl = true
    // ): Observable<any> {
    // 	const formData = new FormData();
    // 	formData.append("data", data);
    // 	formData.append("filename", file, fileName);
    // 	return this.http.post(this.fullUrl(api, useBaseUrl), formData, {
    // 		headers: {}
    // 	});
    // }
    // public uploadFileWithProgressEvent(
    // 	api: string,
    // 	file: File,
    // 	fileName: string,
    // 	useBaseUrl = true
    // ): Observable<any> {
    // 	return Observable.create(observer => {
    // 		const formData: FormData = new FormData(),
    // 			xhr: XMLHttpRequest = new XMLHttpRequest();
    // 		formData.append("files", file, fileName);
    // 		xhr.onreadystatechange = () => {
    // 			if (xhr.readyState === 4) {
    // 				if (xhr.status === 200) {
    // 					observer.next(JSON.parse(xhr.response));
    // 					observer.complete();
    // 				} else {
    // 					observer.error(xhr.response);
    // 				}
    // 			}
    // 		};
    // 		xhr.upload.onprogress = event => {
    // 			const progress = Math.round(event.loaded / event.total * 100);
    // 			this.progressEvent.next(progress);
    // 		};
    // 		xhr.open("POST", this.fullUrl(api, useBaseUrl), true);
    // 		xhr.send(formData);
    // 	});
    // }
    // private functions
    /**
     * @private
     * @param {?} api
     * @param {?} useBaseUrl
     * @return {?}
     */
    WebApiService.prototype.fullUrl = 
    // public uploadFile(
    // 	api: string,
    // 	file: File,
    // 	fileName: string,
    // 	data: string,
    // 	useBaseUrl = true
    // ): Observable<any> {
    // 	const formData = new FormData();
    // 	formData.append("data", data);
    // 	formData.append("filename", file, fileName);
    // 	return this.http.post(this.fullUrl(api, useBaseUrl), formData, {
    // 		headers: {}
    // 	});
    // }
    // public uploadFileWithProgressEvent(
    // 	api: string,
    // 	file: File,
    // 	fileName: string,
    // 	useBaseUrl = true
    // ): Observable<any> {
    // 	return Observable.create(observer => {
    // 		const formData: FormData = new FormData(),
    // 			xhr: XMLHttpRequest = new XMLHttpRequest();
    // 		formData.append("files", file, fileName);
    // 		xhr.onreadystatechange = () => {
    // 			if (xhr.readyState === 4) {
    // 				if (xhr.status === 200) {
    // 					observer.next(JSON.parse(xhr.response));
    // 					observer.complete();
    // 				} else {
    // 					observer.error(xhr.response);
    // 				}
    // 			}
    // 		};
    // 		xhr.upload.onprogress = event => {
    // 			const progress = Math.round(event.loaded / event.total * 100);
    // 			this.progressEvent.next(progress);
    // 		};
    // 		xhr.open("POST", this.fullUrl(api, useBaseUrl), true);
    // 		xhr.send(formData);
    // 	});
    // }
    // private functions
    /**
     * @private
     * @param {?} api
     * @param {?} useBaseUrl
     * @return {?}
     */
    function (api, useBaseUrl) {
        return this.config.baseUrl && useBaseUrl
            ? this.config.baseUrl + api
            : api;
    };
    /**
     * @param {?} apiError
     * @return {?}
     */
    WebApiService.prototype.errorHandler = /**
     * @param {?} apiError
     * @return {?}
     */
    function (apiError) {
        if (apiError.apiUrl) {
            this.errorSubject.next(apiError);
        }
        return throwError(apiError);
    };
    WebApiService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    WebApiService.ctorParameters = function () { return [
        { type: HttpClient },
        { type: LocalStorageService },
        { type: Router },
        { type: WebApiConfig, decorators: [{ type: Optional }] }
    ]; };
    return WebApiService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var JsonFileService = /** @class */ (function () {
    function JsonFileService(http) {
        this.http = http;
        this.errorSubject = new Subject();
        this.jsonFileError = this.errorSubject.asObservable();
    }
    /**
     * @template T
     * @param {?} jsonFileUrl
     * @return {?}
     */
    JsonFileService.prototype.get = /**
     * @template T
     * @param {?} jsonFileUrl
     * @return {?}
     */
    function (jsonFileUrl) {
        return this.getRemoteJsonFile(jsonFileUrl);
    };
    /**
     * @private
     * @template T
     * @param {?} jsonFileUrl
     * @return {?}
     */
    JsonFileService.prototype.getRemoteJsonFile = /**
     * @private
     * @template T
     * @param {?} jsonFileUrl
     * @return {?}
     */
    function (jsonFileUrl) {
        var _this = this;
        return this.http.get(jsonFileUrl).pipe(map((/**
         * @param {?} result
         * @return {?}
         */
        function (result) { return (/** @type {?} */ (result)); })), catchError((/**
         * @param {?} error
         * @return {?}
         */
        function (error) { return _this.errorResponseHandler(error, jsonFileUrl); })));
    };
    /**
     * @private
     * @param {?} error
     * @param {?} jsonFileUrl
     * @return {?}
     */
    JsonFileService.prototype.errorResponseHandler = /**
     * @private
     * @param {?} error
     * @param {?} jsonFileUrl
     * @return {?}
     */
    function (error, jsonFileUrl) {
        /** @type {?} */
        var jsonFileError = (/** @type {?} */ ({}));
        jsonFileError.fileUrl = jsonFileUrl;
        jsonFileError.error = error;
        return throwError(jsonFileError);
    };
    JsonFileService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    JsonFileService.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    return JsonFileService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TimeLoggingInterceptor = /** @class */ (function () {
    function TimeLoggingInterceptor() {
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    TimeLoggingInterceptor.prototype.intercept = /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    function (request, next) {
        /** @type {?} */
        var started = Date.now();
        /** @type {?} */
        var ok;
        return next.handle(request).pipe(tap((
        // Succeeds when there is a response; ignore other events
        // Succeeds when there is a response; ignore other events
        /**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            return (ok = event instanceof HttpResponse ? "succeeded" : "");
        }), (
        // Operation failed; error is an HttpErrorResponse
        // Operation failed; error is an HttpErrorResponse
        /**
         * @param {?} error
         * @return {?}
         */
        function (error) { return (ok = "failed"); })), 
        // Log when response observable either completes or errors
        finalize((/**
         * @return {?}
         */
        function () {
            /** @type {?} */
            var elapsed = Date.now() - started;
            /** @type {?} */
            var msg = "%c " + request.method + " \"" + request.urlWithParams + "\" " + ok + " in " + elapsed + " ms.";
            console.log(msg, "color: #c01025");
        })));
    };
    TimeLoggingInterceptor.decorators = [
        { type: Injectable }
    ];
    return TimeLoggingInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ErrorHandlingInterceptor = /** @class */ (function () {
    function ErrorHandlingInterceptor() {
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    ErrorHandlingInterceptor.prototype.intercept = /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    function (request, next) {
        var _this = this;
        return next.handle(request).pipe(catchError((/**
         * @param {?} errorResponse
         * @param {?} caught
         * @return {?}
         */
        function (errorResponse, caught) {
            console.log("raw error : ", errorResponse);
            return _this.errorHandler(request, errorResponse);
        })));
    };
    /**
     * @private
     * @param {?} request
     * @param {?} errorResponse
     * @return {?}
     */
    ErrorHandlingInterceptor.prototype.errorHandler = /**
     * @private
     * @param {?} request
     * @param {?} errorResponse
     * @return {?}
     */
    function (request, errorResponse) {
        /** @type {?} */
        var apiError = (/** @type {?} */ ({
            method: request.method.toUpperCase(),
            apiUrl: request.url,
            error: errorResponse
        }));
        switch ((/** @type {?} */ (errorResponse.status))) {
            case HTTP_STATUS_CODE.NOT_FOUND:
                return this.error404Handler(apiError);
            case HTTP_STATUS_CODE.SERVER_ERROR:
                return this.error500Handler(apiError);
            default:
                return throwError(apiError);
        }
    };
    /**
     * @private
     * @param {?} apiError
     * @return {?}
     */
    ErrorHandlingInterceptor.prototype.error404Handler = /**
     * @private
     * @param {?} apiError
     * @return {?}
     */
    function (apiError) {
        return throwError(apiError);
    };
    /**
     * @private
     * @param {?} apiError
     * @return {?}
     */
    ErrorHandlingInterceptor.prototype.error500Handler = /**
     * @private
     * @param {?} apiError
     * @return {?}
     */
    function (apiError) {
        console.log("Please handle 500 error - internal server error");
        return throwError(apiError);
    };
    ErrorHandlingInterceptor.decorators = [
        { type: Injectable }
    ];
    return ErrorHandlingInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ContentTypeInterceptor = /** @class */ (function () {
    function ContentTypeInterceptor() {
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    ContentTypeInterceptor.prototype.intercept = /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    function (request, next) {
        if (request.method === "POST" && !(request.body instanceof FormData)) {
            /** @type {?} */
            var headerSet = !!request.headers.get("Content-Type");
            if (!headerSet) {
                request = request.clone({
                    setHeaders: {
                        "Content-Type": "application/json"
                    }
                });
            }
        }
        return next.handle(request);
    };
    ContentTypeInterceptor.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    ContentTypeInterceptor.ctorParameters = function () { return []; };
    return ContentTypeInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TokenInterceptor = /** @class */ (function () {
    function TokenInterceptor(localStorage, config) {
        this.localStorage = localStorage;
        this.config = config;
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    TokenInterceptor.prototype.intercept = /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    function (request, next) {
        /** @type {?} */
        var token = this.localStorage.get(this.config.tokenKey);
        if (token) {
            request = request.clone({
                setHeaders: {
                    token: token
                }
            });
        }
        return next.handle(request);
    };
    TokenInterceptor.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    TokenInterceptor.ctorParameters = function () { return [
        { type: LocalStorageService },
        { type: WebApiConfig, decorators: [{ type: Optional }] }
    ]; };
    return TokenInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var DEFAULT_TIMEOUT = 10000 * 2.5;
var TimeOutInterceptor = /** @class */ (function () {
    function TimeOutInterceptor(config) {
        this.config = config;
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    TimeOutInterceptor.prototype.intercept = /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    function (request, next) {
        /** @type {?} */
        var timeOutTime = this.config.timeout || DEFAULT_TIMEOUT;
        return next.handle(request).pipe(timeout(timeOutTime));
    };
    TimeOutInterceptor.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    TimeOutInterceptor.ctorParameters = function () { return [
        { type: WebApiConfig, decorators: [{ type: Optional }] }
    ]; };
    return TimeOutInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ResultHandlingInterceptor = /** @class */ (function () {
    function ResultHandlingInterceptor() {
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    ResultHandlingInterceptor.prototype.intercept = /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    function (request, next) {
        return next.handle(request).pipe(map((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            if (event instanceof HttpResponse &&
                request.url.indexOf(".json") < 0) {
                if (event.body) {
                    return event.clone({ body: event.body.result });
                }
            }
            return event;
        })));
    };
    ResultHandlingInterceptor.decorators = [
        { type: Injectable }
    ];
    return ResultHandlingInterceptor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var WebApiInterceptorProviders = [
    {
        provide: HTTP_INTERCEPTORS,
        useClass: TimeLoggingInterceptor,
        multi: true
    },
    {
        provide: HTTP_INTERCEPTORS,
        useClass: ResultHandlingInterceptor,
        multi: true
    },
    {
        provide: HTTP_INTERCEPTORS,
        useClass: ErrorHandlingInterceptor,
        multi: true
    },
    { provide: HTTP_INTERCEPTORS, useClass: TimeOutInterceptor, multi: true },
    { provide: HTTP_INTERCEPTORS, useClass: TokenInterceptor, multi: true },
    {
        provide: HTTP_INTERCEPTORS,
        useClass: ContentTypeInterceptor,
        multi: true
    } // adding content type to headers
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var TryCatchWebApiModule = /** @class */ (function () {
    function TryCatchWebApiModule(parentModule) {
        if (parentModule) {
            throw new Error("TryCatch Web Api Module is already loaded. Import it in the AppModule only");
        }
    }
    /**
     * @param {?} config
     * @return {?}
     */
    TryCatchWebApiModule.forRoot = /**
     * @param {?} config
     * @return {?}
     */
    function (config) {
        return {
            ngModule: TryCatchWebApiModule,
            providers: [
                WebApiService,
                { provide: WebApiConfig, useValue: config }
            ]
        };
    };
    TryCatchWebApiModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        HttpClientModule,
                        TrycatchLocalStorageModule.forRoot({
                            storageType: "local"
                        })
                    ],
                    providers: [WebApiInterceptorProviders, WebApiService, JsonFileService]
                },] }
    ];
    /** @nocollapse */
    TryCatchWebApiModule.ctorParameters = function () { return [
        { type: TryCatchWebApiModule, decorators: [{ type: Optional }, { type: SkipSelf }] }
    ]; };
    return TryCatchWebApiModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { HTTP_STATUS_CODE, HTTP_METHOD, WebApiConfig, WebApiService, JsonFileService, TryCatchWebApiModule, ContentTypeInterceptor as ɵg, ErrorHandlingInterceptor as ɵd, ResultHandlingInterceptor as ɵc, TimeLoggingInterceptor as ɵb, TimeOutInterceptor as ɵe, TokenInterceptor as ɵf, WebApiInterceptorProviders as ɵa };

//# sourceMappingURL=trycatch-web-api.js.map